//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
/**
@file         expansion.cpp
@brief        Example: How to expand LxS_Lib with new features
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

#include <iostream>
#include <fstream>
using namespace std;

#include "LxS_Lib.h"
#pragma comment(lib, "LxS_Lib.lib") // LxS Library
LxS::Device * lxs = NULL;

bool do_save_profile = false;
int profile_count = 0;
ofstream file;

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       static void Execute_Example_001(void)
@brief    Example 001
@details
Example 001 shows how to get "Measure-Data" from sensor with Raw- and Lib-methods.
*/
static void Execute_Example_001(void)
{
  cout << endl << "Example 001: Switch to MEASURE-Mode and take 100 Profiles" << endl;
  file.open("..\\output.txt", ios::out | ios::trunc);
  file << "Profile;Scan-Number;Encoder-Value;Type;Data;" << endl;
  do_save_profile = true; // Data-Handler-ON
  // ------------------------------------------------------------------------------------
  cout << endl << "a) with Raw-Methods, created from \"Technical Description\"" << endl;
  (void)lxs->RawExecuteCommand(((LxS::eCmd)(0x3133)), NULL, 0, true, ((LxS::eCmd)(0x4141))); // LxS::LXS_CMD_EXIT_COMMAND_MODE
  Sleep(1000); // save data with callback function while waiting for 1 sec.
  (void)lxs->RawExecuteCommand(((LxS::eCmd)(0x3132)), NULL, 0, true, ((LxS::eCmd)(0x4141))); // LxS::LXS_CMD_ENTER_COMMAND_MODE
  cout << endl;
  // ------------------------------------------------------------------------------------
  cout << endl << "b) with Lib-Method \"SetSensorMode(...)\"" << endl;
  (void)lxs->SetSensorMode(LxS::LXS_MODE_MEASURE);
  Sleep(1000); // save data with callback function while waiting for 1 sec.
  (void)lxs->SetSensorMode(LxS::LXS_MODE_COMMAND);
  do_save_profile = false; // Data-Handler-OFF
  file.flush(); file.close(); cout << endl << endl;
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       static void Execute_Example_002(void)
@brief    Example 002
@details
Example 002 shows how to Control Sensor true Raw and Lib Methods with Arguments.
*/
static void Execute_Example_002(void)
{
  cout << endl << "Example 002: Switch Laser-Gate on/off 10 Times" << endl;
  // ------------------------------------------------------------------------------------
  cout << endl << "a) with Raw-Methods, created from \"Technical Description\"" << endl;
  bool state = false;
  // Create Message-Buffer for Argument
  BYTE * msg = new BYTE[2];     // unmanaged : create buffer
  msg[0] = 0x00; msg[1] = 0x00; // initialisation ... (false)
  for(int i = 0; i < 10; i++)
  {
    // Set Argument (16bit, LSB)
    msg[0] = ((state) ? (0x01) : (0x00)); msg[1] = 0x00;
    // "Common"-Method to communicate with LxS-device "ExecuteCommand(...)"
    // LXS_CMD_SET_LASER_GATE = 0x0001 with Argument and Argument-Length (in Bytes)
    bool done = lxs->RawExecuteCommand(((LxS::eCmd)(0x0001)), msg, 2, true, ((LxS::eCmd)(0x4141))); // LxS::LXS_CMD_SET_LASER_GATE
    state = !state; Sleep(500); cout << ((done) ? (".") : ("!"));
  };
  cout << endl;
  // ------------------------------------------------------------------------------------
  cout << endl << "b) with Lib-Method \"SetLaserGate(...)\"" << endl;
  for (int i = 0; i < 10; i++)
  {
    bool done = lxs->SetLaserGate(state);
    state = !state; Sleep(500); cout << ((done) ? (".") : ("!"));
  };
  cout << endl << endl;
  // Free unmanaged Ressources
  delete[] msg; // unmanaged : free buffer
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       static void CALLBACK MyCB(LxS::eData dt, BYTE * ptr, int len)
@brief    Data-Handler
@param    dt - Type of Package
@param    ptr - Pointer to Data-Field
@param    len - Length of Data-Filed
@details
Data-Handler will be called from Lib after receiving new Measure-Data from Sensor.
*/
static void CALLBACK MyCB(LxS::eData dt, BYTE * ptr, int len)
{
  if (do_save_profile) // if Output-File for Data-Handler created ...
  {                    // Send Package-Information to File ...
    cout << ".";
    file << (++profile_count) << ";" << lxs->GetScanNumber() << ";" << lxs->GetEncoderValue() << ";" << dt << ";";
    if ((ptr != NULL) && (len > 0))
    {
      char * conv = new char[len * 2 + 1];
      for(int i = 0; i < len; i++) { sprintf_s(&conv[i*2], 3, "%02X", ptr[i]); };
      conv[len * 2] = 0; file << conv; delete[] conv;
    };
    file << ";" << endl;
  };
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       int main(void)
@brief    Main
@details
Main-Function create Connection to LxS-device and running Examples.
*/
int main(void)
{
  cout << "Start Expansion-Example" << endl;
  // Constructor create new instance of LxS-device
  lxs = new LxS::Device();
  // Data-Handler will be called after Lib receives new measure from LxS-device
  lxs->SetDataHandler(MyCB);
  // Create new UDP-socket for LxS-device
  if (lxs->Create())
  {
    // TimeOut set to 2s
    (void)lxs->SetTimeOut(2000);
    // Connect to LxS-device
    if (lxs->Connect())
    {
      // Switch to COMMAND-Mode, turn Laser-ON
      (void)lxs->SetSensorMode(LxS::LXS_MODE_COMMAND);
      (void)lxs->SetLaserGate(true);

      cout << "Created, Connected and in COMMAND-Mode" << endl;

      // Run Examples ...
      Execute_Example_001();
      Execute_Example_002();

      // Switch to MEASURE-Mode
	  (void)lxs->SetSensorMode(LxS::LXS_MODE_MEASURE);
      // Disconnect from LxS-device
      (void)lxs->Disconnect();
    }
    else
    {
      cout << "Error ! Connect failed" << endl;
    };

    // Release UDP-Socket
    (void)lxs->Destroy();
  }
  else
  {
    cout << "Error ! Create failed" << endl;
  };

  // Destroy instance, free Ressources
  lxs->~Device();

  cout << "Done. Press any key and carriage return to quit ..." << endl;
  int inp = getchar();
  return (0);
}
