//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
//--------------------------------------------------------------------------------------------------------------------
/**
@file         LxS_Device.h
@brief        Definition of a Class that represents a LxS-device
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

#ifndef   __DEVEL_LXS_DEVICE_H__
#define   __DEVEL_LXS_DEVICE_H__

//---------------------------------------------------------------------------------------------------------------------
// common interface
//
#include "LxS_Lib.h"

//---------------------------------------------------------------------------------------------------------------------
#ifdef    __cplusplus
extern "C"
{
#endif // __cplusplus

  //! Main Namespace for LxS-Lib
  namespace LxS
  {
    //-----------------------------------------------------------------------------------------------------------------
    /**
    @class      Device
    @brief      Class that represents a virtual LxS-device
    @details    In this class are all functions to control an LxS sensor over ethernet interface.
    */
    class DLLSPEC   Device
    {
    protected:
      bool          is_created;                   //!< Socket was created
      bool          is_connected;                 //!< Socket was connected to port and listening
      bool          is_running;                   //!< Thread is running
      bool          is_high;                      //!< marks a High-Resolution-device
      int           thread_count;                 //!< (internal) step counting

    private:
      bool          is_threaded;                  //!< LxS should be threaded
      ADDR_IN       dll_end_point;                //!< Socket settings for PC-site
      ADDR_IN       lxs_end_point;                //!< Socket settings for LxS-device
      WSADATA       wsa;                          //!< Windows sockets structure
      SOCKET        sock;                         //!< Main socket
      THREAD        thread;                       //!< Main thread
      CallBackFunc  cb_func;                      //!< CallBack function for DataHandler

      int           time_out;                     //!< TimeOut for request/response
      int           pkg_num;                      //!< Package counting
      bool          do_send;                      //!< Flag, represent request state
      BYTE *        send_pkg;                     //!< Request package
      int           send_len;                     //!< Length of request package
      BYTE          in_buffer[LXS_IN_BUFFER_LEN]; //!< Buffer for incoming packages
      bool          has_resp;                     //!< Flag, represent response state
      eCmd          resp_cmd;                     //!< Expected response command
      BYTE *        resp_pkg;                     //!< Response package
      int           resp_len;                     //!< Length of response package
      bool          async_sync;                   //!< Synchronisation for Async-Waiting

      eMode         last_mode;                    //!< Sensor-Mode from last incoming package
      UINT16        last_state;                   //!< Sensor-State from last incoming package
      UINT16        last_scan;                    //!< Scan-Number from last incoming package
      UINT32        last_encoder;                 //!< Encoder-Value from last incoming package
      bool          last_activation;              //!< Activation-Flag from last incoming package
      bool          last_ethernet;                //!< Ethernet-Flag from last incoming package
      bool          last_warning;                 //!< Warning-Flag from last incoming package
      bool          last_error;                   //!< Error-Flag from last incoming package
      clock_t       last_trigger;                 //!< Time of last trigger event
      bool          last_EthActivState;           //!< Activation state set by ethernet activation command

      LxS::eCmd     internal_GetResponseCmd(LxS::eCmd cmd);
      bool          internal_CreateSocket(void);
      bool          internal_StartThread(void);
      BYTE *        internal_CreatePackage(eCmd cmd, BYTE * msg = NULL, int len = 0);
      void          internal_IncomingPackage(int len);
      BYTE *        internal_GetBytes(BYTE * bts, int offset, int len);
      ParamSet *    internal_GetParamSet(int id, int dtype);
      bool          internal_SetParamSet(int id, int dtype, ParamSet * prm, bool permanent = false);
      static void   LxS_Thread(void * param) { THREAD_BLOCK(param); }

    public:
      Device(int port = LXS_DEFAULT_SERV, bool threaded = true);
      ~Device();
      void          OneThreadStep(void);
      void          SetDataHandler(CallBackFunc func);
      bool          SetTimeOut(int time);
      bool          Create(const char * addr = LXS_DEFAULT_ADDR, int port = LXS_DEFAULT_PORT);
      bool          Destroy(void);
      bool          Connect(bool high = false);
      bool          Disconnect(void);
      bool          ExecuteCommand(eCmd cmd, BYTE * msg = NULL, int len = 0, bool blocking = true);
      bool          RawExecuteCommand(eCmd cmd, BYTE * msg = NULL, int len = 0, bool blocking = true, eCmd answerCmd = LXS_RESP_ACK_SUCCESS);
      UINT16        GetStatus(void);
      bool          SetLaserGate(bool state);
      bool          EthernetTrigger(bool blockingFlag = false, BYTE cmdTrigRespMask = 7, UINT16 cmdTriggerWait = 49, UINT16 minWaitNonBlockingMode = 20);
      bool          SetSensorMode(eMode mode);
      eMode         GetSensorMode(void);
      bool          SetScanNumber(UINT16 scan);
      UINT16        GetScanNumber(void);
      bool          SetEncoderValue(UINT32 enc);
      UINT32        GetEncoderValue(void);
      bool          SetInspectionTask(UINT16 taskNum, bool permanent = false);
      UINT16        GetInspectionTask(void);
      bool          SetActivation(bool state);
      bool          GetActivation(void);
      eType         GetDeviceType(void);
      const char *  GetDeviceName(eType type);
      Config *      GetDeviceConfiguration(void);
      bool          SetDeviceConfiguration(Config * cfg, bool permanent = true);
      ParamSet *    GetTaskSettings(UINT16 taskNum);
      bool          SetTaskSettings(UINT16 taskNum, ParamSet * prm, bool permanent = false);
      const char *  GetErrorText(UINT16 err_no);
      Param *       GetSingleTaskParam(UINT16 param_id);
      bool          SetSingleTaskParam(UINT16 param_id, Param * prm = NULL, bool permanent = false);
      Param *       GetSingleUserParam(UINT16 param_id);
      bool          SetSingleUserParam(UINT16 param_id, Param * prm = NULL, bool permanent = false);
      const char *  GetVersion(void);
      UINT16 *      ReadResponsePackage(UINT16 * len);
    };
  }

#ifdef    __cplusplus
}
#endif // __cplusplus

#endif // __DEVEL_LXS_DEVICE_H__
