//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
//--------------------------------------------------------------------------------------------------------------------
/**
@file         LxS_ParamSet.cpp
@brief        Implementation of a Class that represents a LxS-Parameter-Set
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// common interface
//
#include "LxS_Lib.h"

//! Main Namespace for LxS-Lib
namespace LxS
{
  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       BYTE * ParamSet::internal_CreateCopy(BYTE * bts, int len)
  @brief    Create Copy
  @param    bts - original pointer
  @param    len - length
  @return   New BYTE-Array
  @details
  Create a Copy from given Array.
  */
  BYTE * ParamSet::internal_CreateCopy(BYTE * bts, int len)
  {
    BYTE * ret = NULL;
    if ((bts != NULL) && (len > 0))
    {
      ret = new BYTE[len]; // unmanaged : create buffer
      memcpy(ret, bts, len); // copy original data to buffer
      //delete[] ret; // unmanaged : free buffer
    };
    //! @remark : user should free ressources with : delete[] ret; after using
    return (ret); //! @remark : !!! UNMANAGED RETURN !!!
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       ParamSet::ParamSet(int id, int stype, BYTE * field, int f_len, BYTE * data, int d_len)
  @brief    Constructor for LxS::ParamSet-Class
  @param    id - parameter id
  @param    stype - structure type
  @param    field - pointer to structure
  @param    f_len - length of structure
  @param    data - pointer to data
  @param    d_len - length of data
  @details
  Create new Instance of LxS::ParamSet-Class from given Structure and Data.
  */
  ParamSet::ParamSet(int id, int stype, BYTE * field, int f_len, BYTE * data, int d_len)
  {
    this->prm_list = NULL;
    this->prm_struct = this->internal_CreateCopy(field, f_len); // unmanaged : copy structure
    this->prm_length = f_len;
    this->id = id;
    this->stype = stype;
    int str_len = (f_len >> 1); // 8bit -> 16bit
    int dt_len = (d_len >> 1); // 8bit -> 16bit
    if ((str_len > 0) && (dt_len > 0)) // if structure and data not empty
    {
      UINT16 * ptr_struct = ((UINT16 *)(field)); // byte* => short*
      UINT16 * ptr_data = ((UINT16 *)(data));    // byte* => short*
      int p = 0, d = 0;
      LxS::Param * act_prm = NULL; // last created Parameter
      while ((str_len > p) && (ptr_struct[p] != 0)) // loop over all structure-entries
      {
        LxS::Param * prm = new LxS::Param(&(ptr_struct[p]), &(ptr_data[d])); // unmanaged : create new parameter
        if (this->prm_list != NULL) // if allready has parameter in the list ...
        {
          act_prm->next = prm; // add new parameter to list
          act_prm = prm;
        }
        else
        {
          this->prm_list = prm; // new parameter on top of list
          act_prm = prm;
        };
        d += ptr_struct[p + 2]; // increment pointers ...
        p += 8;
      };
    };
    //! @remark : user should free ressources with : prm->~Param(); after using
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       ParamSet::~ParamSet()
  @brief    Destructor for LxS::ParamSet-Class
  @details
  Method killing a Instance of LxS::ParamSet-Class.
  */
  ParamSet::~ParamSet()
  {
    if (this->prm_list != NULL) // cascade to all entries in set
    {
      this->prm_list->~Param(); // destructor
    };
    delete[] this->prm_struct; // free own data
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       LxS::Param * ParamSet::GetSettings(void)
  @brief    Returns Parameter-Set
  @return   A list with parameters
  @details
  Method returns a Parameter-Set as List of Parameters.
  */
  LxS::Param * ParamSet::GetSettings(void)
  {
    return (this->prm_list);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       bool ParamSet::CheckStructure(LxS::ParamSet * prm)
  @brief    Compare own and given Structures
  @return   Both Structurea are same
  @details
  Compare own and given Structures.
  */
  bool ParamSet::CheckStructure(LxS::ParamSet * prm)
  {
    bool ret = false;
    if ((prm != NULL) && (this->prm_struct != NULL))
    {
      if ((prm->stype == this->stype) && (prm->id == this->id)) // same id and type ?
      {
        if (prm->prm_length == this->prm_length) // same length ?
        {
          ret = true;
          for(int i = 0; i < this->prm_length; i++) // for each element
          {
            if (this->prm_struct[i] != prm->prm_struct[i]) // struct same ?
            {
              ret = false; // if different => so not same !
              break;
            };
          };
        };
      };
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       BYTE * ParamSet::GenerateParameterPackage(int * len)
  @brief    Create a Package for all Parameters in a Parameter-Set
  @param    len - (out) length of generated package
  @return   Package with all Parameter-Data
  @details
  Create a Package for all Parameters in a Parameter-Set.
  */
  BYTE * ParamSet::GenerateParameterPackage(int * len)
  {
    BYTE * ret = NULL;
    int cnt = 0;
    int pkg_len = 0;
    LxS::Param * ptr = this->prm_list;  // set pointer to first parameter
    while (ptr != NULL) // for all elements
    {
      pkg_len += ptr->cnt; // calculate total length
      ptr = ptr->next; // next element
    };
    if (pkg_len > 0) // if parameter-set not empty ...
    {
      int p = 4;
      ptr = this->prm_list; // set pointer to first parameter
      cnt = (pkg_len << 1) + 4; // length
      ret = new BYTE[cnt]; // unmanaged : create buffer
      ret[0] = ((BYTE)(this->id)); ret[1] = ((BYTE)(this->id >> 8)); // set id
      ret[2] = 0x00; ret[3] = 0x00; // set 0x0000
      while (ptr != NULL) // for all elements
      {
        int c2 = (ptr->cnt << 1);
        memcpy(&(ret[p]), ptr->data, c2); // copy parameter data to buffer
        p += c2;
        ptr = ptr->next; // next element
      };
    };
    *len = cnt; // return length of package
    //! @remark : user should free ressources with : delete[] ret; after using
    return (ret); //! @remark : !!! UNMANAGED RETURN !!!
  }
}
