//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
//--------------------------------------------------------------------------------------------------------------------
/**
@file         native.cpp
@brief        Native Handling for LxS-Lib
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// common interface
//
#include "LxS_Lib.h"

//! Native-Interface for LxS-Lib
namespace Native
{
  //---------------------------------------------------------------------------------------------------------------------
  /**
  @struct     sNativWrapper
  @brief      User-Defined-Datatype, represents a Data-Set for Nativ Applications without DataHandling abilities
  */
  typedef struct
  {
    UINT16  scan_nr;        //!< Scan-Number
    UINT16  data_x[400];    //!< X-Data
    UINT16  x_len;          //!< Length of X-Data
    UINT16  data_z[400];    //!< Z-Data
    UINT16  z_len;          //!< Length of Z-Data
    UINT16  data_f[400];    //!< FIELD-Data
    UINT16  f_len;          //!< Length of FIELD-Data

  } sWrapper;
  static sWrapper my_wrap;  //!< statical temporary data-set for DataHandling

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       static void CALLBACK MyNativCB(LxS::eData dt, BYTE * ptr, int len)
  @brief    (internal) DataHandler for LxS-CallBacks
  @param    dt - Type of Data-Set
  @param    ptr - Data-Set
  @param    len - Length of Data-Set
  @details
  Internal DataHandler will accumulate last Data-Set for Applications without CallBack-abilities.
  */
  static void CALLBACK MyNativCB(LxS::eData dt, BYTE * ptr, int len)
  {
    UINT16 * ptr_data1 = NULL;
    UINT16 * ptr_len1 = NULL;
    UINT16 * ptr_data2 = NULL;
    UINT16 * ptr_len2 = NULL;
    UINT16 * ptr_data3 = NULL;
    UINT16 * ptr_len3 = NULL;
    switch (dt)
    {
    case LxS::LXS_DATA_Z:
      {
        ptr_data1 = my_wrap.data_z;
        ptr_len1 = &my_wrap.z_len;
        break;
      }
    case LxS::LXS_DATA_X:
      {
        ptr_data1 = my_wrap.data_x;
        ptr_len1 = &my_wrap.x_len;
        break;
      }
    case LxS::LXS_DATA_ZX:
      {
        ptr_data1 = my_wrap.data_z;
        ptr_len1 = &my_wrap.z_len;
        ptr_data2 = my_wrap.data_x;
        ptr_len2 = &my_wrap.x_len;
        break;
      }
    case LxS::LXS_DATA_FIELD:
      {
        ptr_data3 = my_wrap.data_f;
        ptr_len3 = &my_wrap.f_len;
        break;
      }
    };
    if ((ptr_data1 != NULL) && (ptr_len1 != NULL))
    {
      if ((ptr_data2 != NULL) && (ptr_len2 != NULL))
      {
        memcpy(ptr_data1, ptr, len/2);
        *ptr_len1 = len/2;
        memcpy(ptr_data2, (ptr + len/2), len/2);
        *ptr_len2 = len/2;
      }
      else
      {
        memcpy(ptr_data1, ptr, len);
        *ptr_len1 = len;
      }
    }
    if ((ptr_data3 != NULL) && (ptr_len3 != NULL))
    {
      memcpy(ptr_data3, ptr, len);
      *ptr_len3 = len;
    }
    delete[] ptr;
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC LxS::Device * CALLBACK CreateInstance(void)
  @brief    Create new Wrapper
  @return   New Instance of LxS::Device
  @details
  Create new Instance of LxS::Device.
  */
  DLLSPEC LxS::Device * CALLBACK CreateInstance(void)
  {
    LxS::Device * ret = NULL;
    try
    {
      //! @todo : !!! PC-Port and High-Connect not implemented !!!
      ret = new LxS::Device();
    }
    catch (...) { ret = NULL; }
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK ReleaseInstance(LxS::Device * lxs)
  @brief    Destroy the Wrapper
  @param    lxs - Pointer to LxS::Device-Instance
  @return   Operational State (Instance was destroyed)
  @details
  Destroy the LxS::Device-Instance and break DataHandler-Events.
  */
  DLLSPEC bool CALLBACK ReleaseInstance(LxS::Device * lxs)
  {
    try
    {
      lxs->SetDataHandler(NULL);
    }
    catch (...) { };
    try
    {
      (void)lxs->Disconnect();
    }
    catch (...) { };
    try
    {
      lxs->~Device();
    }
    catch (...) { };
    try
    {
      delete lxs;
    }
    catch (...) { }
    return (true);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK ConnectDevice(LxS::Device * lxs, const char * addr, int port)
  @brief    Connect to LxS::Device
  @param    lxs - Pointer to LxS::Device-Instance
  @param    addr - LxS-IP-Address
  @param    port - LxS-Port
  @return   Operational State (was connected ...)
  @details
  Connect to LxS::Device and set (internal) DataHandler
  */
  DLLSPEC bool CALLBACK ConnectDevice(LxS::Device * lxs, const char * addr, int port)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      try
      {
        if (lxs->Create(addr, port))
        {
          if (lxs->Connect())
          {
            lxs->SetDataHandler(MyNativCB);
            ret = true;
          };
        };
      }
      catch (...) { ret = false; };
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK DisconnectDevice(LxS::Device * lxs)
  @brief    Disconnect from LxS::Device
  @param    lxs - Pointer to LxS::Device-Instance
  @return   Operational State (was disconnected ...)
  @details
  Disconnect to LxS::Device and reset (internal) DataHandler
  */
  DLLSPEC bool CALLBACK DisconnectDevice(LxS::Device * lxs)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      try
      {
        lxs->SetDataHandler(NULL);
        ret = (lxs->Disconnect());
      }
      catch (...) { ret = false; };
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK SetDeviceLaserGate(LxS::Device * lxs, bool state)
  @brief    Wrapper: lxs->SetLaserGate(state);
  @param    lxs - Pointer to LxS::Device-Instance
  @param    state - New Laser-State
  @return   Operational State (was set ...)
  @details
  Wrapper for lxs->SetLaserGate(state);
  */
  DLLSPEC bool CALLBACK SetDeviceLaserGate(LxS::Device * lxs, bool state)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->SetLaserGate(state);
      }
      catch (...) { ret = false; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK SetDeviceToCommandMode(LxS::Device * lxs, bool mode)
  @brief    Wrapper: lxs->SetSensorMode(mode);
  @param    lxs - Pointer to LxS::Device-Instance
  @param    mode - New Device-Mode
  @return   Operational State (was set ...)
  @details
  Wrapper for lxs->SetSensorMode(mode);
  */
  DLLSPEC bool CALLBACK SetDeviceToCommandMode(LxS::Device * lxs, bool mode)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->SetSensorMode((mode) ? (LxS::LXS_MODE_COMMAND) : (LxS::LXS_MODE_MEASURE));
      }
      catch (...) { ret = false; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC UINT16 CALLBACK GetDeviceScanNumber(LxS::Device * lxs)
  @brief    Wrapper: lxs->GetScanNumber();
  @param    lxs - Pointer to LxS::Device-Instance
  @return   Scan-Number
  @details
  Wrapper for lxs->GetScanNumber();
  */
  DLLSPEC UINT16 CALLBACK GetDeviceScanNumber(LxS::Device * lxs)
  {
    UINT16 ret = 0xFFFF;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->GetScanNumber();
      }
      catch (...) { ret = 0xFFFF; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC UINT32 CALLBACK GetDeviceEcoderValue(LxS::Device * lxs)
  @brief    Wrapper: lxs->GetEncoderValue();
  @param    lxs - Pointer to LxS::Device-Instance
  @return   Encoder-Value
  @details
  Wrapper for lxs->GetEncoderValue();
  */
  DLLSPEC UINT32 CALLBACK GetDeviceEcoderValue(LxS::Device * lxs)
  {
    UINT32 ret = 0xFFFFFFFF;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->GetEncoderValue();
      }
      catch (...) { ret = 0xFFFFFFFF; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC UINT16 CALLBACK GetDeviceInspectionTask(LxS::Device * lxs)
  @brief    Wrapper: lxs->GetInspectionTask();
  @param    lxs - Pointer to LxS::Device-Instance
  @return   Inspektion-Task
  @details
  Wrapper for lxs->GetInspectionTask();
  */
  DLLSPEC UINT16 CALLBACK GetDeviceInspectionTask(LxS::Device * lxs)
  {
    UINT16 ret = 0xFFFF;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->GetInspectionTask();
      }
      catch (...) { ret = 0xFFFF; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK SetDeviceInspectionTask(LxS::Device * lxs, UINT16 task)
  @brief    Wrapper: lxs->SetInspectionTask(task);
  @param    lxs - Pointer to LxS::Device-Instance
  @param    task - New Inspektion-Task
  @return   Inspektion-Task
  @details
  Wrapper for lxs->SetInspectionTask(task);
  */
  DLLSPEC bool CALLBACK SetDeviceInspectionTask(LxS::Device * lxs, UINT16 task)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->SetInspectionTask(task);
      }
      catch (...) { ret = false; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK DeviceEthernetTrigger(LxS::Device * lxs)
  @brief    Wrapper: lxs->EthernetTrigger();
  @param    lxs - Pointer to LxS::Device-Instance
  @return   Operational State (was executed ...)
  @details
  Wrapper for lxs->EthernetTrigger();
  */
  DLLSPEC bool CALLBACK DeviceEthernetTrigger(LxS::Device * lxs)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      try
      {
        ret = lxs->EthernetTrigger();
      }
      catch (...) { ret = false; }
    };
    return (ret);
  }

  //-------------------------------------------------------------------------------------------------------------------
  /**
  @fn       DLLSPEC bool CALLBACK DeviceDataSet(LxS::Device * lxs, sWrapper * wrap)
  @brief    The given Data-Set will be filled with last Data-Set
  @param    lxs - Pointer to LxS::Device-Instance
  @param    wrap - (out) Wraper-UDT to Application
  @return   Operational State (was executed ...)
  @details
  The given Data-Set will be filled with last Data-Set for Applications without CalBack-Handling
  */
  DLLSPEC bool CALLBACK DeviceDataSet(LxS::Device * lxs, sWrapper * wrap)
  {
    bool ret = false;
    if (lxs != NULL)
    {
      wrap->scan_nr = lxs->GetScanNumber();
      wrap->x_len = (my_wrap.x_len >> 1);
      wrap->z_len = (my_wrap.z_len >> 1);
      wrap->f_len = (my_wrap.f_len >> 1);
      if (my_wrap.x_len > 0)
      {
        memcpy(wrap->data_x, my_wrap.data_x, (my_wrap.x_len));
      };
      if (my_wrap.z_len > 0)
      {
        memcpy(wrap->data_z, my_wrap.data_z, (my_wrap.z_len));
      };
      if (my_wrap.f_len > 0)
      {
        memcpy(wrap->data_f, my_wrap.data_f, (my_wrap.f_len));
      };
      ret = true;
    };
    return (ret);
  }
}
