//=====================================================================================================================
// This code is from Leuze electronic GmbH + Co. KG. It is free software, WITHOUT ANY GUARANTEES OR WARRANTY
// (without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE).
// Although the author has attempted to find and correct any bugs in this free software, the author is not responsible
// for any damage or losses of any kind caused by the use or misuse of the program. 
// The author is under no obligation to provide support, service, corrections, or upgrades to this software.
/**
@file         tester.cpp
@brief        Console-Tester for LxS-Lib with Examples of using.
@author       Archipov Slava
@version      1.0.0
*/
//=====================================================================================================================

#include <iostream>
using namespace std;
#include <string>

#include "LxS_Lib.h"
#pragma comment(lib, "LxS_Lib.lib") // LxS Library
LxS::Device * lxs = NULL;


//-------------------------------------------------------------------------------------------------------------------
/**
@fn       static void CALLBACK MyCB(LxS::eData dt, BYTE * ptr, int len)
@brief    Data-Handler
@param    dt - Type of Package
@param    ptr - Pointer to Data-Field
@param    len - Length of Data-Filed
@details
Data-Handler will be called from Lib after receiving new Measure-Data from Sensor.
*/
static void CALLBACK MyCB(LxS::eData dt, BYTE * ptr, int len)
{
  cout << "Data Handler : " << len << " : " << lxs->GetScanNumber() << " : " << lxs->GetEncoderValue() << endl;
}


//-------------------------------------------------------------------------------------------------------------------
/**
@fn       static void PrintHexData(BYTE * buf, int len)
@brief    Print-Bytes as HEX
@param    buf - Pointer to Data-Field
@param    len - Length of Data-Filed
@details
Function will print Byte-Field to console as HEX
*/
static void PrintHexData(BYTE * buf, int len)
{
  if ((buf != NULL) && (len > 0))
  {
    char * conv = new char[len * 2 + 1];
    for(int i = 0; i<len; i++)
    {
      sprintf_s(&conv[i*2], 3, "%02X", buf[i]);
    }
    conv[len * 2] = 0;
    cout << conv << endl;
    delete[] conv;
  };
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       static void PrintPrmNext(LxS::Param * p, int cnt)
@brief    Print-Parameter
@param    p - Pointer to parameter
@param    cnt - Number
@details
Function will print parameter list to console
*/
static void PrintPrmNext(LxS::Param * p, int cnt)
{
  if (p != NULL)
  {
    cout << cnt << " : " << p->id << " : " << p->dt << " : " << p->cnt << " : " << p->min << " : " << p->max << " : " << p->flds << " : ";
    PrintHexData((BYTE *)p->data, (p->cnt << 1));
    PrintPrmNext(p->next, cnt + 1);
  }
  else
  {
    cout << cnt << " : NULL" << endl;
  };
}

//-------------------------------------------------------------------------------------------------------------------
/**
@fn       int main(void)
@brief    Main
@details
Main-Function create connection to LxS-device and running tests and examples.
*/
int main(void)
{
  cout << "Start LxS-Lib Tester" << endl;

  cout << "Constructor ..." << endl;
  lxs = new LxS::Device();

  cout << "Set Data-Handler ..." << endl;
  lxs->SetDataHandler(MyCB);

  cout << "Call create ..." << endl;
  if (lxs->Create())
  {
    cout << "Set TimeOut to 2sec ... " << (lxs->SetTimeOut(2000) ? ("ok") : ("NO !!!")) << endl;

    cout << "Call Connect ... ";

    if (lxs->Connect())
    {
      cout << "Switch Sensor to COMMAND-MODE ..." << (lxs->SetSensorMode(LxS::LXS_MODE_COMMAND) ? ("ok") : ("NO !!!")) << endl;
      cout << "Get Single Task Param 3003(Activation mode) ..." << endl;
      LxS::Param * x = lxs->GetSingleTaskParam(3003);
      PrintPrmNext(x, 0);
      x->data[0] = 1;
      cout << "Set Single Task Param 3003(Activation mode) ..." << (lxs->SetSingleTaskParam(3003, x) ? ("ok") : ("NO !!!")) << endl;
      x->~Param();
      cout << "Switch sensor to MEASURE-MODE ..." << (lxs->SetSensorMode(LxS::LXS_MODE_MEASURE) ? ("ok") : ("NO !!!")) << endl;
      cout << "Wait 1 sec ..." << endl;
      Sleep(1000);
      // Toogle 3 times activation state in "MEASUE MODE" with ethernet command.
      for(int cnt = 0; cnt < 3; cnt++)
      {
        cout << "Set activation (false) ..." << (lxs->SetActivation(false) ? ("ok") : ("NO !!!")) << endl;
        cout << "Get activation ..." << "actual: " << (lxs->GetActivation()) << endl;
        cout << "Wait 0,2 sec ..." << endl;
        Sleep(200);
        cout << "Set activation (true) ..." << (lxs->SetActivation(true) ? ("ok") : ("NO !!!")) << endl;
        cout << "Get activation ..." << "actual: " << (lxs->GetActivation()) << endl;
        cout << "Wait 0,2 sec ..." << endl;
        Sleep(200);
      }

      cout << "Switch Sensor to COMMAND-MODE ..." << (lxs->SetSensorMode(LxS::LXS_MODE_COMMAND) ? ("ok") : ("NO !!!")) << endl;

      // Toggle laser gate state 20 times.
      cout << "Switch laser state ..." << endl;
      for (int i = 0; i < 20; i++)
      {
        cout << (lxs->SetLaserGate(true) ? ("1") : ("X"));
        Sleep(20);
        cout << (lxs->SetLaserGate(false) ? ("0") : ("X"));
        Sleep(20);
      };
      (void)lxs->SetLaserGate(true);
      cout << endl;

      cout << "Get Inspection-Task ... ";
      UINT16 it = lxs->GetInspectionTask();
      cout << "actual is : " << it << endl;
      it++;
      cout << "Set inspection task to (" << it << ") ... " << (lxs->SetInspectionTask(it) ? ("ok") : ("NO !!!")) << endl;
      cout << "Get inspection task (again) ... " << "actual: " << (lxs->GetInspectionTask()) << endl;

      LxS::eType ty = lxs->GetDeviceType();
      cout << "Get device type: " << ty << " : " << (lxs->GetDeviceName(ty)) << endl;

      cout << "Get task settings ..." << endl;
      LxS::ParamSet * ps = lxs->GetTaskSettings(0);
      PrintPrmNext(ps->GetSettings(), 0);
      cout << "Set task settings ... " << (lxs->SetTaskSettings(1, ps) ? ("ok") : ("NO !!!")) << endl;
      ps->~ParamSet(); // unmanaged

      cout << "Get single task parameter 3002(operation mode) ..." << endl;
      LxS::Param * opMode = lxs->GetSingleTaskParam(3002);
      if (opMode != NULL)
      {
        PrintPrmNext(opMode, 0);
        opMode->data[0] = 1; // set to input trigger
        cout << "Set single task parameter 3002(operation mode) => Input triggered ..." << (lxs->SetSingleTaskParam(3002, opMode) ? ("ok") : ("NO !!!")) << endl;
      };

      cout << ">>> Ethernet Trigger in COMMAND-Mode ! <<<" << endl;
      for (int i = 0; i < 50; i++)
      {
        (void)lxs->EthernetTrigger(true, 4, 0); Sleep(1);
      };

      cout << "Get single task parameter 3002(operation mode) ..." << endl;
      opMode = lxs->GetSingleTaskParam(3002);
      if (opMode != NULL)
      {
        PrintPrmNext(opMode, 0);
        opMode->data[0] = 0; // set to input trigger
        cout << "Set single task parameter 3002(operation mode) => Free running ..." << (lxs->SetSingleTaskParam(3002, opMode) ? ("ok") : ("NO !!!")) << endl;
        opMode->~Param();
      };


      cout << "Get error messages ..." << endl;
      for (int i = 100; i < 110; i++)
      {
        const char * ptr = lxs->GetErrorText(i);
        cout << i << " : " << ptr << endl;
        delete[] ptr;
      };

      cout << "Set new scan number ..." << (lxs->SetScanNumber(7777) ? ("ok") : ("NO !!!")) << endl;

      cout << "Set new encoder value ..." << (lxs->SetEncoderValue(55667788) ? ("ok") : ("NO !!!")) << endl;

      cout << "Get single task parameter 3001(inspection task name) ..." << endl;
      LxS::Param * data1 = lxs->GetSingleTaskParam(3001);
      PrintPrmNext(data1, 0);
	  // Write same parameter values as read before
      cout << "Set single task parameter 3001(inspection task name) ..." << (lxs->SetSingleTaskParam(3001, data1) ? ("ok") : ("NO !!!")) << endl;
      data1->~Param();

      cout << "Get single user parameter 2011(median filter flag) ..." << endl;
      LxS::Param * y = lxs->GetSingleUserParam(2011);
      PrintPrmNext(y, 0);
	  // Write same parameter values as read before
      cout << "Set single user parameter 2011(median filter flag) ..." << (lxs->SetSingleUserParam(2011, y) ? ("ok") : ("NO !!!")) << endl;
      y->~Param();



      cout << "Change Exposure ..." << endl;
      LxS::Param * z = lxs->GetSingleTaskParam(3006);
      if (z != NULL)
      {
	    // Increase exposure time all 100 ms.
        for (int i = 2; i < 40; i++)
        {
          int val = ((i == 2) ? (739) : (((i - 3) * 328) + 973));
          val = ((val > (int)z->max) ? ((int)z->max) : (val));
          val = ((val < (int)z->min) ? ((int)z->min) : (val));
          z->data[0] = ((UINT16)(val));
          cout << (lxs->SetSingleTaskParam(3006, z, false) ? ("+") : ("X"));
          Sleep(100);
        };
	    // Decrease exposure time all 100 ms.
        for (int j = 2; j < 40; j++)
        {
          int i = 42 - j;
          int val = ((i == 2) ? (739) : (((i - 3) * 328) + 973));
          val = ((val > (int)z->max) ? ((int)z->max) : (val));
          val = ((val < (int)z->min) ? ((int)z->min) : (val));
          z->data[0] = ((UINT16)(val));
          cout << (lxs->SetSingleTaskParam(3006, z, false) ? ("-") : ("X"));
          Sleep(100);
        };
        z->~Param();
        cout << endl;
      }
      else
      {
        cout << "ERROR ON READ 3006 !!!" << endl;
      };


      cout << "Get configuration ...";
      LxS::Config * cfg = lxs->GetDeviceConfiguration();
      cout << ((cfg != NULL) ? ("ok") : ("NO !!!")) << endl;
      if (cfg != NULL)
      {
        cout << "GENERAL : " << ((cfg->general != NULL) ? ("ok") : ("NO !!!")) << endl;
        if (cfg->general != NULL) { PrintPrmNext(cfg->general->GetSettings(), 0); }
        cout << "CALIBRATION : " << ((cfg->calibration != NULL) ? ("ok") : ("NO !!!")) << endl;
        if (cfg->calibration != NULL) { PrintPrmNext(cfg->calibration->GetSettings(), 0); }
        cout << "TEMP : " << ((cfg->temp != NULL) ? ("ok") : ("NO !!!")) << endl;
        if (cfg->temp != NULL) { PrintPrmNext(cfg->temp->GetSettings(), 0); }
        for (int i = 0; i < 16; i++)
        {
          cout << "TASK (" << i << ") : " << ((cfg->task[i] != NULL) ? ("ok") : ("NO !!!")) << endl;
          if (cfg->task[i] != NULL) { PrintPrmNext(cfg->task[i]->GetSettings(), 0); }
        };
      };
      cout << "Set configuration ... " << ((lxs->SetDeviceConfiguration(cfg, false)) ? ("ok") : ("NO !!!")) << endl;
      cfg->~Config();

      cout << "Get single task parameter 3002(operation mode) ..." << endl;
      LxS::Param * data2 = lxs->GetSingleTaskParam(3002);
      if (data2 != NULL)
      {
        PrintPrmNext(data2, 0);
        data2->data[0] = 1; // set to input trigger
        cout << "Set single task parameter 3002(operation mode) => Input triggered ..." << (lxs->SetSingleTaskParam(3002, data2) ? ("ok") : ("NO !!!")) << endl;
        data2->~Param();
      };

      cout << "Get single user parameter 2003(task selection inputs) ..." << endl;
      LxS::Param * data3 = lxs->GetSingleUserParam(2003);
      if (data3 != NULL)
      {
        PrintPrmNext(data3, 0);
        data3->data[0] = 0; // Disable input for task selection
        cout << "Set single user parameter 2003(task selection inputs) => Disable..." << (lxs->SetSingleUserParam(2003, data3) ? ("ok") : ("NO !!!")) << endl;
        data3->~Param();
      };


      cout << "Switch sensor to MEASURE-MODE ..." << (lxs->SetSensorMode(LxS::LXS_MODE_MEASURE) ? ("ok") : ("NO !!!")) << endl;

      cout << "ok ... wait 50ms" << endl;
      Sleep(50);
      cout << ">>> Ethernet Trigger in MEASURE-MODE ! <<<" << endl;
      int cnt = 0;
      cout << "1. Trigger 50 times (unblocked with fix waiting time)" << endl;
      for (int i = 0; i < 50; i++)
      {
        // unblocked with fix waiting time
        (void)lxs->EthernetTrigger();
        cnt++;
      };
      cout << "2. Trigger 50 times (blocked, wait for first telegram)" << endl;
      for (int i = 0; i < 50; i++)
      {
        // blocked, wait for first telegram
        (void)lxs->EthernetTrigger(true);
        cnt++;
      };
      cout << "Switch Sensor to COMMAND-MODE ..." << (lxs->SetSensorMode(LxS::LXS_MODE_COMMAND) ? ("ok") : ("NO !!!")) << endl;
      cout << "Set inspection task back to 0 ... " << (lxs->SetInspectionTask(0) ? ("ok") : ("NO !!!")) << endl;
      BYTE * parameter = new BYTE[4]; // unmanaged : create buffer
      //Parameter1 = 25 (Offset = 25 [mm])
      parameter[0] = 25;  // Lowbyte
      parameter[1] = 0;   // Highbyte
      //Parameter2 = 3 (Sensitivity = 3 [coarse])
      parameter[2] = 3;   // Lowbyte
      parameter[3] = 0;   // Highbyte 

      cout << "Execute basic area scan ... " << ((lxs->ExecuteCommand(LxS::LXS_CMD_EXECUTE_AREA_SCAN_BASIC_TEACH, parameter, 4, true)) ? ("ok") : ("NO !!!")) << endl;
      delete[] parameter;
      UINT16 len;
      UINT16 * data = lxs->ReadResponsePackage(&len);
      if(len == 2)
      {
        cout << "Response Data (Error Num): " << data[0] << endl;
        cout << "Response Data (Teach Value1): " << data[1] << endl;
      }
      cout << "Switch sensor to MEASURE-MODE ..." << (lxs->SetSensorMode(LxS::LXS_MODE_MEASURE) ? ("ok") : ("NO !!!")) << endl;

      cout << "Call Disconnect ... " << (lxs->Disconnect() ? ("ok") : ("NO !!!")) << endl;
    }
    else
    {
      cout << "Error ! Connect command failed" << endl;
    };

    cout << "Call Destroy ... " << (lxs->Destroy() ? ("ok") : ("NO !!!")) << endl;
  }
  else
  {
    cout << "Error ! Create failed" << endl;
  };

  cout << "Destructor ..." << endl;
  lxs->~Device();

  cout << "Done. Press any key and carriage return to quit ..." << endl;
  int inp = getchar();
  return (0);
}
